@@ -41,20 +41,18 @@ typedef struct {
  * Team Port Settings
  */
 struct _NMSettingTeamPort {
-    NMSetting parent;
-    /* In the past, this struct was public API. Preserve ABI! */
+    NMSetting                parent;
+    NMSettingTeamPortPrivate _priv;
 };
 
 struct _NMSettingTeamPortClass {
     NMSettingClass parent;
-    /* In the past, this struct was public API. Preserve ABI! */
-    gpointer padding[4];
 };
 
 G_DEFINE_TYPE(NMSettingTeamPort, nm_setting_team_port, NM_TYPE_SETTING)
 
 #define NM_SETTING_TEAM_PORT_GET_PRIVATE(o) \
-    (G_TYPE_INSTANCE_GET_PRIVATE((o), NM_TYPE_SETTING_TEAM_PORT, NMSettingTeamPortPrivate))
+    _NM_GET_PRIVATE(o, NMSettingTeamPort, NM_IS_SETTING_TEAM_PORT, NMSetting)
 
 /*****************************************************************************/
 
@@ -313,7 +311,7 @@ verify(NMSetting *setting, NMConnection *connection, GError **error)
 
     if (connection) {
         NMSettingConnection *s_con;
-        const char *         slave_type;
+        const char          *slave_type;
 
         s_con = nm_connection_get_setting_connection(connection);
         if (!s_con) {
@@ -400,12 +398,12 @@ duplicate_copy_properties(const NMSettInfoSetting *sett_info, NMSetting *src, NM
 }
 
 static gboolean
-init_from_dbus(NMSetting *                     setting,
-               GHashTable *                    keys,
-               GVariant *                      setting_dict,
-               GVariant *                      connection_dict,
+init_from_dbus(NMSetting                      *setting,
+               GHashTable                     *keys,
+               GVariant                       *setting_dict,
+               GVariant                       *connection_dict,
                guint /* NMSettingParseFlags */ parse_flags,
-               GError **                       error)
+               GError                        **error)
 {
     guint32  changed = 0;
     gboolean success;
@@ -426,7 +424,7 @@ init_from_dbus(NMSetting *                     setting,
 static void
 get_property(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
-    NMSettingTeamPort *       setting = NM_SETTING_TEAM_PORT(object);
+    NMSettingTeamPort        *setting = NM_SETTING_TEAM_PORT(object);
     NMSettingTeamPortPrivate *priv    = NM_SETTING_TEAM_PORT_GET_PRIVATE(setting);
 
     switch (prop_id) {
@@ -457,10 +455,10 @@ get_property(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 static void
 set_property(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
-    NMSettingTeamPort *       setting = NM_SETTING_TEAM_PORT(object);
+    NMSettingTeamPort        *setting = NM_SETTING_TEAM_PORT(object);
     NMSettingTeamPortPrivate *priv    = NM_SETTING_TEAM_PORT_GET_PRIVATE(setting);
     guint32                   changed;
-    const GPtrArray *         v_ptrarr;
+    const GPtrArray          *v_ptrarr;
 
     switch (prop_id) {
     case NM_TEAM_ATTRIBUTE_CONFIG:
@@ -528,11 +526,9 @@ finalize(GObject *object)
 static void
 nm_setting_team_port_class_init(NMSettingTeamPortClass *klass)
 {
-    GObjectClass *  object_class        = G_OBJECT_CLASS(klass);
+    GObjectClass   *object_class        = G_OBJECT_CLASS(klass);
     NMSettingClass *setting_class       = NM_SETTING_CLASS(klass);
-    GArray *        properties_override = _nm_sett_info_property_override_create_array();
-
-    g_type_class_add_private(klass, sizeof(NMSettingTeamPortPrivate));
+    GArray         *properties_override = _nm_sett_info_property_override_create_array();
 
     object_class->get_property = get_property;
     object_class->set_property = set_property;
@@ -702,5 +698,5 @@ nm_setting_team_port_class_init(NMSettingTeamPortClass *klass)
                              NM_META_SETTING_TYPE_TEAM_PORT,
                              NULL,
                              properties_override,
-                             NM_SETT_INFO_PRIVATE_OFFSET_FROM_CLASS);
+                             G_STRUCT_OFFSET(NMSettingTeamPort, _priv));
 }
